--

with System;

generic
   type Item_Type is private;

package Bounded_Buffers is
   pragma Elaborate_Body;

   Default_Ceiling : constant System.Priority := System.Default_Priority;

   type Buffer_Type (Buffer_Size : Positive; Ceiling : System.Priority) is limited private;

   procedure Put (Buf : in out Buffer_Type; Item : in Item_Type);
   procedure Get (Buf : in out Buffer_Type; Item : out Item_Type);

   function Is_Full (Buf : Buffer_Type) return Boolean;

private

   type Item_Array_Type is array (Positive range <>) of Item_Type;

   -- -- --

   protected type Buffer_Type (Buffer_Size : Positive; Ceiling : System.Priority) is
      pragma Priority (Ceiling);

      entry Put (Item : in Item_Type);
      entry Get (Item : out Item_Type);

      function Is_Full return Boolean;

   private
      Buffer_Store : Item_Array_Type (1 .. Buffer_Size);

      Next_In  : Positive := 1;
      Next_Out : Positive := 1;

      Count : Natural := 0;
   end Buffer_Type;

end Bounded_Buffers;
